# MIT License
#
# Copyright (c) 2023 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import TermTk as ttk
import ttkDesigner.app.superobj as so
from .superobj import SuperObject


class SuperWidgetTextEdit(so.SuperWidget):
    @staticmethod
    def _swFromWidget(wid, swClass, *args, **kwargs):
        return swClass(wid=wid, *args, **kwargs)

    def getSuperProperties(self):
        additions, exceptions, exclude = super().getSuperProperties()
        exclude += ['Layout']
        return additions, exceptions, exclude

    def dumpDict(self):
        wid = self._wid
        ret = {
            'class'  : wid.__class__.__name__,
            'params' : SuperObject.dumpParams(wid,exclude=['Layout']),
        }
        return ret


# MIT License
#
# Copyright (c) 2023 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from random import randint

import TermTk as ttk

import ttkDesigner.app.superobj as so
from .superobj import SuperObject

class SuperWidget(ttk.TTkContainer):
    __slots__ = ('_wid', '_superRootWidget', '_designer',
                 'superMoved', 'superResized')
    def __init__(self, designer, wid, *args, **kwargs):
        self.superMoved   = ttk.pyTTkSignal(int,int)
        self.superResized = ttk.pyTTkSignal(int,int)
        self._designer = designer
        self._wid = wid
        self._wid.move(*kwargs['pos'])
        self._wid._canvas.show()
        self._superRootWidget = kwargs.get('superRootWidget',False)
        kwargs['maxSize'] = wid.maximumSize()
        kwargs['minSize'] = wid.minimumSize()
        kwargs['size']    = wid.size()
        super().__init__(*args, **kwargs)
        #self.resize(*self._wid.size())
        h,s,l = randint(0,359),100,randint(60,80)
        r,g,b = ttk.TTkColor.hsl2rgb(((h+5)%360,s,l))
        self._layoutColor    = ttk.TTkColor.bg(f"#{r:02X}{g:02X}{b:02X}", modifier=ttk.TTkColorGradient(increment=+2))
        r,g,b = ttk.TTkColor.hsl2rgb(((h+5)%360,s,l))
        self._layoutPadColor = ttk.TTkColor.bg(f"#{r:02X}{g:02X}{b:02X}", modifier=ttk.TTkColorGradient(increment=-2))
        self.setFocusPolicy(ttk.TTkK.ClickFocus)
        SuperWidget.toggleHighlightLayout.connect(self._toggleHighlightLayout)

    _showLayout = False
    toggleHighlightLayout = ttk.pyTTkSignal(bool)

    # TODO: Find a better way to handle this exception
    # It may require some major rewrite
    def hasControlWidget(self):
        return True

    @ttk.pyTTkSlot(str)
    def setSuperName(self, name):
        if name and name not in [w.name() for w in self._designer.getWidgets()]:
            oldName = self._wid._name
            self._wid.setName(name)
            self._designer.widgetNameChanged.emit(oldName, name)
            self._designer.weModified.emit()

    def getSuperProperties(self):
        additions = {}
        exceptions = {
            'Name': {
                'get':  { 'cb':ttk.TTkWidget.name,               'type':str} ,
                'set':  { 'cb':lambda _,n: self.setSuperName(n), 'type':str} },
        }
        exclude = []
        return additions, exceptions, exclude

    @ttk.pyTTkSlot(bool)
    def _toggleHighlightLayout(self, state):
        SuperWidget._showLayout = state
        self.update()

    def dumpDict(self):
        wid = self._wid
        ret = {
            'class'  : wid.__class__.__name__,
            'params' : SuperObject.dumpParams(wid),
        }
        return ret

    @staticmethod
    def _swFromWidget(wid, swClass, *args, **kwargs):
        sw = swClass(wid=wid, *args, **kwargs)
        return sw

    @staticmethod
    def swFromWidget(wid:object, *args, **kwargs):
        swClass = so.SuperWidget
        for c, sc in {
            # ttk.TTkTextEdit:       so.SuperWidgetTextEdit,
            ttk.TTkRadioButton:    so.SuperWidgetRadioButton,
            # ttk.TTkResizableFrame: so.SuperWidgetFrame,
            # ttk.TTkWindow:         so.SuperWidgetFrame,
            ttk.TTkSplitter:       so.SuperWidgetSplitter,
            # ttk.TTkList:           so.SuperWidgetList,
            ttk.TTkMenuButton:     so.SuperWidgetMenuButton,
            ttk.TTkFrame:          so.SuperWidgetFrame,
            ttk.TTkAbstractScrollArea: so.SuperWidgetAbstractScrollArea,
            ttk.TTkContainer:      so.SuperWidgetContainer,
            ttk.TTkWidget:         so.SuperWidget,
                }.items():
            if c in type(wid).mro():
                swClass = sc
                break
        return swClass._swFromWidget(wid=wid, swClass=swClass, *args, **kwargs)

    @staticmethod
    def loadDict(designer, parent, widProp):
        ttkClass = getattr(ttk,widProp['class'])
        if issubclass(ttkClass,ttk.TTkLayout):
            demiProp = {
                'version':'2.0.0',
                'tui':{
                    'class'  : widProp['class'],
                    'params' : widProp['params'],
                    'children' : []
                },
                'connections':[]
            }
            layout = ttk.TTkUiLoader.loadDict(demiProp)
            x,y,w,h = layout.geometry()
            sl = sup = so.SuperLayout.slFromLayout(designer=designer, layout=layout, lay=ttk.TTkLayout(), pos=(x,y), size=(w,h), selectable=True)
            children = widProp['children']
        elif issubclass(ttkClass,ttk.TTkContainer):
            setLayout  = 'layout'  in widProp
            setMenuBar = 'menuBar' in widProp
            tui = {
                'class'  : widProp['class'],
                'params' : widProp['params'] }
            tui |= {
                'layout': {
                    'class'  : widProp['layout']['class'],
                    'params' : widProp['layout']['params'],
                    'children' : []
                    } } if setLayout else {}
            tui |= {
                'menuBar': widProp['menuBar']
                    } if setMenuBar else {}
            demiProp = {
                'version':'2.0.0',
                'tui': tui,
                'connections':[]
            }
            wid = ttk.TTkUiLoader.loadDict(demiProp)
            sup = so.SuperWidgetContainer.swFromWidget(designer=designer, wid=wid, pos=wid.pos())
            sl = sup._superLayout
            children = widProp['layout']['children'] if setLayout else []
        else:
            setMenuBar = 'menuBar' in widProp
            tui = {
                'class'  : widProp['class'],
                'params' : widProp['params'] }
            demiProp = {
                'version':'2.0.0',
                'tui': tui,
                'connections':[]
            }
            wid = ttk.TTkUiLoader.loadDict(demiProp)
            sup = so.SuperWidget.swFromWidget(designer=designer, wid=wid, pos=wid.pos())
            children = []

        for ch in children:
            sch = SuperWidget.loadDict(designer, parent, ch)
            if issubclass(type(sch),so.SuperLayout):
                schl = sch
            elif issubclass(type(sch),so.SuperWidgetContainer):
                schl = sch._superLayout
            else:
                schl = None

            if issubclass(type(sl),so.SuperLayoutGrid):
                sl.layout().addWidget(sch,ch['row'],ch['col'],ch['rowspan'],ch['colspan'])
                if schl:
                    schl.setDropBorder(1)
            else:
                sl.layout().addWidget(sch)
                if schl:
                    schl.setDropBorder(0)
        return sup

    def updateAll(self):
        self.resize(*(self._wid.size()))
        self.move(*(self._wid.pos()))
        self.setMaximumSize(*(self._wid.maximumSize()))
        self.setMinimumSize(*(self._wid.minimumSize()))
        self.update()

    def mousePressEvent(self, evt) -> bool:
        return True

    def isRootWidget(self):
        return self._superRootWidget

    def makeRootWidget(self):
        self._superRootWidget = True

    def pushSuperControlWidget(self):
        # if self._superRootWidget: return False
        scw = so.SuperControlWidget(self)
        ttk.TTkHelper.removeOverlay()
        ttk.TTkHelper.overlay(self, scw, -1,-1, forceBoundaries=False)

    def mouseReleaseEvent(self, evt) -> bool:
        w,h = self.size()
        if 0<=evt.x<w and 0<=evt.y<h:
            self.pushSuperControlWidget()
            self._designer.thingSelected.emit(self._wid,self)
        return True

    def mouseDragEvent(self, evt) -> bool:
        if self._superRootWidget: return False
        drag = ttk.TTkDrag()
        data = self
        data.paintChildCanvas()
        drag.setHotSpot(evt.x, evt.y)
        drag.setPixmap(data.getCanvas())
        drag.setData(data)
        drag.exec()
        self.parentWidget().removeSuperWidget(self)
        return True

    def superChild(self):
        return self._wid

    def move(self, x: int, y: int):
        self._wid.move(x,y)
        self.update()
        self.superMoved.emit(x,y)
        return super().move(x, y)

    def resizeEvent(self, w, h):
        self._wid.resize(w,h)
        self._wid._canvas.updateSize()
        self.superResized.emit(w,h)
        return super().resizeEvent(w, h)

    def paintEvent(self, canvas):
        w,h = self.size()
        if SuperWidget._showLayout:
            for y in range(h):
                canvas.drawText(pos=(0,y),text='',width=w,color=self._layoutColor)
            for y in range(0,h):
                canvas.drawText(pos=(0,y),text='',width=w,color=self._layoutPadColor)
            # canvas.fill(color=self._layoutColor)
            # canvas.fill(pos=(l,t), size=(w-r-l,h-b-t), color=self._layoutPadColor)
        else:
            self._wid.getCanvas().updateSize()
            self._wid.paintEvent(self._wid.getCanvas())
            canvas.paintCanvas(
                    self._wid.getCanvas(),
                    (    0,     0, w, h), # geometry
                    (    0,     0, w, h), # slice
                    (    0,     0, w, h)) # bound


# MIT License
#
# Copyright (c) 2023 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os

import TermTk as ttk
from TermTk.TTkCore.canvas import TTkCanvas

from .superobj.superwidgetmenubutton import SuperWidgetMenuButton


class _MenuItem(ttk.TTkWidget):
    ''' Generic Control Button for the MenuEditor

        [Menu[+][x]

    '''
    __slots__ = ('_text', '_lineEdit', '_autoResize', '_menuButton', '_superMenuButton', '_designer',
                 # Signals
                 'closeClicked')
    def __init__(self, menuButton, designer, autoResize=True, *args, **kwargs):
        self._text = ttk.TTkString(menuButton.text())
        self._menuButton = menuButton
        self._autoResize = autoResize
        self._designer = designer
        self._superMenuButton = SuperWidgetMenuButton.factoryGetSuperWidgetMenuButton(wid=menuButton, designer=self._designer)
        self.closeClicked  = ttk.pyTTkSignal(_MenuItem)
        super().__init__(*args, **(kwargs|{'size':(self._text.termWidth()+7,1)}))
        self.processWidgetName(self._menuButton.name())
        self._lineEdit = ttk.TTkLineEdit(parent=self, visible=False, text=self._text)
        self._lineEdit.returnPressed.connect(self._lineEdit.hide)
        self._lineEdit.textEdited.connect(self._textEdited)
        self._lineEdit.focusChanged.connect(self._lineEdit.setVisible)

    def processWidgetName(self, name):
        names = {w.name() for w in self._designer.getWidgets() if w is not self._menuButton}
        index = 1
        className = name
        while className in names:
            className = f"{name}-{index}"
            index += 1
        self._menuButton.setName(className)

    @ttk.pyTTkSlot(str)
    def _textEdited(self, text):
        self._text = text
        self._menuButton.setText(text)
        width = text.termWidth()
        if self._autoResize:
            self.resize(width+7,1)
            self._lineEdit.setGeometry(1,0,width,1)
        self.processWidgetName(f"menu_{text}")
        self._designer.weModified.emit()
        self.update()

    def mouseDoubleClickEvent(self, evt) -> bool:
        w,h = self.size()
        if evt.x <= w-7:
            self._lineEdit.setText(self._text)
            self._lineEdit.setGeometry(1,0,w-7,1)
            self._lineEdit.show()
            self._lineEdit.setFocus()
        return True

    def expandMenuItem(self):
        subMenuEditor = _SubMenuEditor(size=(20,10), menuButton=self._menuButton, designer=self._designer)
        subMenuEditor.itemsChanged.connect(self.update)
        wi = subMenuEditor.widgetItem()
        wi.setLayer(wi.LAYER1)
        w = self.width()
        ttk.TTkHelper.overlay(self, subMenuEditor, w-3, 0)

    def mouseReleaseEvent(self, evt) -> bool:
        w = self.width()
        if evt.x > w-4:
            self.closeClicked.emit(self)
        elif evt.x > w-7:
            self.expandMenuItem()
        else:
            self._designer.thingSelected.emit(self._menuButton,self._superMenuButton)
        return True

    def paintEvent(self, canvas: TTkCanvas):
        w = self.width()
        text = (
                ttk.TTkString( "[",        ttk.TTkColor.fg("#FFFF66"))+
                ttk.TTkString( self._text, ttk.TTkColor.RST))
        canvas.drawText(text=text)
        expandIcon = ">" if len(self._menuButton._submenu)> 0 else "+"
        text = (
                ttk.TTkString( "[",        ttk.TTkColor.fg("#AAAA44"))+
                ttk.TTkString( expandIcon, ttk.TTkColor.fg("#00FF00"))+
                ttk.TTkString( "][",       ttk.TTkColor.fg("#AAAA44"))+
                ttk.TTkString( "x",        ttk.TTkColor.fg("#FF0000"))+
                ttk.TTkString( "]",        ttk.TTkColor.fg("#FFFF66")))
        canvas.drawText(text=text, pos=(w-6,0))


class _SubMenuSpacer(ttk.TTkWidget):
    ''' Generic Control Splitter for the MenuEditor

        ---------[x]

    '''
    __slots__ = ('closeClicked', '_menuButton')
    def __init__(self, menuSpacer, *args, **kwargs):
        self._menuButton = menuSpacer
        self.closeClicked  = ttk.pyTTkSignal(_MenuItem)
        super().__init__(*args, **kwargs)

    def mouseReleaseEvent(self, evt) -> bool:
        w = self.width()
        if evt.x > w-4:
            self.closeClicked.emit(self)
        return True

    def paintEvent(self, canvas):
        w = self.width()
        canvas.drawText(pos=(0,0), text="-"*self.width())
        text = (
                ttk.TTkString( "[",       ttk.TTkColor.fg("#AAAA44"))+
                ttk.TTkString( "x",        ttk.TTkColor.fg("#FF0000"))+
                ttk.TTkString( "]",        ttk.TTkColor.fg("#FFFF66")))
        canvas.drawText(text=text, pos=(w-3,0))


class _SubMenuAreaWidget(ttk.TTkAbstractScrollView):
    '''
        ┌────────┤Left├────────╥────────
        │╿+╿[menu[>]┌──────────────────┐
        │╽+╽<XXXXXXX│[menu1      [+][x]│
        └───────────│[menu2      [+][x]│
        ════════════│---------------[x]│
                    │[menu3      [+][x]│
        ────────────│[menu4      [+][x]│
                    │[    Add Menu    ]│
        ────────────│[   Add Spacer   ]│
        ────────────│                  │
                    └──────────────────┘
    '''
    __slots__ = ('_items', '_menuButton', '_minWith',
                 '_btnAddSpacer', '_btnAddMenu', '_designer',
                 #Signals
                 'itemsChanged')
    def __init__(self, menuButton, designer, **kwargs):
        self.itemsChanged = ttk.pyTTkSignal(list)
        self._items = []
        self._designer = designer
        self._menuButton = menuButton
        self._minWidth = 0
        self._btnAddSpacer = ttk.TTkButton(text="Add Spacer")
        self._btnAddMenu   = ttk.TTkButton(text="Add Menu")
        super().__init__(**kwargs)
        self.layout().addWidget(self._btnAddSpacer)
        self.layout().addWidget(self._btnAddMenu  )
        self.setFocusPolicy(ttk.TTkK.ClickFocus + ttk.TTkK.TabFocus)
        self.viewChanged.connect(self._viewChangedHandler)
        self._btnAddSpacer.clicked.connect(self.addSpacer)
        self._btnAddMenu.clicked.connect(self.addMenu)
        for item in self._menuButton._submenu:
            self._importMenuItem(item)

    def close(self):
        self.itemsChanged.clear()
        return super().close()

    def _resizeEvent(self):
        w,h = self.size()
        w = max(w,self._minWidth)
        for i,wid in enumerate(self._items):
            wid.setGeometry(0,i,w,1)
        yy = len(self._items)
        self._btnAddMenu.setGeometry(  0, yy,   w, 1)
        self._btnAddSpacer.setGeometry(0, yy+1, w, 1)

    def resizeEvent(self, w, h):
        self._resizeEvent()

    def _importMenuItem(self,item):
        if issubclass(type(item),ttk.TTkMenuButton):
            item = _MenuItem(menuButton=item, autoResize=True, designer=self._designer)
        else:
            item = _SubMenuSpacer(menuSpacer=item)
        self._items.append(item)
        self._addMenuItem(item)

    @ttk.pyTTkSlot(_MenuItem)
    def removeMenuItem(self, item):
        self._items.pop(self._items.index(item))
        self.layout().removeWidget(item)
        self._menuButton.removeMenuItem(item._menuButton)
        item.closeClicked.disconnect(self.removeMenuItem)
        self._resizeEvent()
        self.itemsChanged.emit(self._items)
        self._designer.weModified.emit()

    def _addMenuItem(self, item):
        item.closeClicked.clear()
        item.closeClicked.connect(self.removeMenuItem)
        self.layout().addWidget(item)
        self._minWidth = max(self._minWidth,item.minimumWidth())
        self._resizeEvent()
        self.itemsChanged.emit(self._items)
        self._designer.weModified.emit()

    def addMenuItem(self, item):
        self._items.append(item)
        self._addMenuItem(item)

    @ttk.pyTTkSlot()
    def addMenu(self):
        mb = self._menuButton.addMenu(text="menu")
        mb.setName("menuButton")
        self.addMenuItem(_MenuItem(menuButton=mb, autoResize=False, designer=self._designer))

    @ttk.pyTTkSlot()
    def addSpacer(self):
        self._menuButton.addSpacer()
        ms = self._menuButton._submenu[-1]
        self.addMenuItem(_SubMenuSpacer(menuSpacer=ms))

    @ttk.pyTTkSlot()
    def _viewChangedHandler(self):
        x,y = self.getViewOffsets()
        self.layout().setOffset(-x,-y)

    def viewFullAreaSize(self) -> tuple:
        _,_,w,h = self.layout().fullWidgetAreaGeometry()
        return w , h

    def viewDisplayedSize(self) -> tuple:
        return self.size()

    def maximumWidth(self):   return 0x10000
    def maximumHeight(self):  return 0x10000
    def minimumWidth(self):   return 0
    def minimumHeight(self):  return 0


class _SubMenuEditor(ttk.TTkResizableFrame):
    __slots__ = ('_scrollView'
                 # Forwarded Signals
                 'itemsChanged')
    def __init__(self, menuButton, designer, **kwargs):
        super().__init__(**kwargs|{'layout':ttk.TTkGridLayout()})
        sa = ttk.TTkScrollArea(parent=self)
        self._scrollView = _SubMenuAreaWidget(menuButton=menuButton, designer=designer)
        sa.setViewport(self._scrollView)

        self.itemsChanged = self._scrollView.itemsChanged


class _MenuBarItemEditorView(ttk.TTkAbstractScrollView):
    '''
        ┌────────┤Left├────────╥──────┤Center├───────╥───────┤Right├───────┐
        │╿+╿[menu[+][x]        ║╿+╿                  ║╿+╿                  │
        │╽+╽                   ║╽+╽                  ║╽+╽                  │
        └──────────────────────╨─────────────────────╨─────────────────────┘
    '''
    __slots__ = ('_itemsLayout', '_items', '_designer')
    def __init__(self, itemsLayout, designer, *args, **kwargs):
        self._items = []
        self._itemsLayout = itemsLayout
        self._designer = designer
        super().__init__(*args, **kwargs)
        self.viewChanged.connect(self._viewChangedHandler)
        for item in self._itemsLayout.children():
            self._importMenuItem(item.widget())

    @ttk.pyTTkSlot()
    def _viewChangedHandler(self):
        x,y = self.getViewOffsets()
        self.layout().setOffset(-x,-y)

    def viewFullAreaSize(self) -> (int, int):
        _,_,w,h = self.layout().fullWidgetAreaGeometry()
        return w, h

    def viewDisplayedSize(self) -> (int, int):
        return self.size()

    def _importMenuItem(self,menuButton):
        item = _MenuItem(menuButton=menuButton, autoResize=True, designer=self._designer)
        self._items.append(item)
        self._addMenuItem(item)

    @ttk.pyTTkSlot(_MenuItem)
    def removeMenuItem(self, item:_MenuItem):
        item.sizeChanged.disconnect(self._refreshItems)
        item.closeClicked.disconnect(self.removeMenuItem)
        self._itemsLayout.removeWidget(item._menuButton)
        self.layout().removeWidget(item)
        self._items.pop(self._items.index(item))
        self._refreshItems()
        self._designer.weModified.emit()

    def _addMenuItem(self, item):
        item.sizeChanged.clear()
        item.closeClicked.clear()
        item.sizeChanged.connect(self._refreshItems)
        item.closeClicked.connect(self.removeMenuItem)
        self.layout().addWidget(item)
        self._refreshItems()
        self._designer.weModified.emit()

    @ttk.pyTTkSlot()
    def addMenuItem(self):
        button = ttk.TTkMenuButton(text="menu", name="menuButton")
        self._itemsLayout.addWidget(button)
        self._importMenuItem(button)

    def _refreshItems(self):
        x = 0
        for mi in self._items:
            w = mi.width()
            mi.move(x,0)
            x+=w
        _,__,w,h = self.layout().fullWidgetAreaGeometry()
        self.resize(w,h)


class _MenuItemEditor(ttk.TTkGridLayout):
    __slots__ = ('_addButton', '_scrollPart','_menuBarView')
    def __init__(self, itemsLayout, designer):
        super().__init__()
        self._addButton =ttk.TTkButton(text="+\n+",border=False, maxWidth=3, minWidth=3, minHeight=2, maxHeight=2)
        self._scrollPart = ttk.TTkAbstractScrollArea(
                verticalScrollBarPolicy   = ttk.TTkK.ScrollBarAlwaysOff ,
                horizontalScrollBarPolicy = ttk.TTkK.ScrollBarAlwaysOn  )
        self._menuBarView = _MenuBarItemEditorView(itemsLayout, designer=designer)
        self._scrollPart.setViewport(self._menuBarView)
        self.addWidget(self._addButton,0,0)
        self.addWidget(self._scrollPart,0,1)
        self._addButton.clicked.connect(self._menuBarView.addMenuItem)


class MenuBarEditor(ttk.TTkWindow):
    __slots__ = ('_editorMenuBar', '_frameOptions', '_widget', '_designer',
                 '_btnTop', '_btnBottom', '_cbTop', '_cbBottom',
                 '_itemsTop', '_itemsBottom', '_mbTop', '_mbBottom')
    def __init__(self, widget:ttk.TTkFrame, designer):
        self._widget = widget
        self._designer = designer
        ttk.TTkUiLoader.loadFile(
            os.path.join(os.path.dirname(os.path.abspath(__file__)),"../tui/menuBarEditor.tui.json"),
            self)
        self._editorMenuBar        = self.getWidgetByName('EditorMenuBar')
        self._cbTop    = self.getWidgetByName('CbTop')
        self._cbBottom = self.getWidgetByName('CbBottom')
        self._btnTop    = self.getWidgetByName('BtnEditTop')
        self._btnBottom = self.getWidgetByName('BtnEditBottom')
        self._frameOptions = self.getWidgetByName("FrameOptions")

        self._btnTop.clicked.connect(   lambda : self._showEditor(ttk.TTkK.TOP))
        self._btnBottom.clicked.connect(lambda : self._showEditor(ttk.TTkK.BOTTOM))

        self._cbTop.clicked.connect(lambda en: self._enableMenuBar(en, ttk.TTkK.TOP))
        self._cbBottom.clicked.connect(lambda en: self._enableMenuBar(en, ttk.TTkK.BOTTOM))

        self._mbTop    = self._widget.menuBar(ttk.TTkK.TOP)
        self._mbBottom = self._widget.menuBar(ttk.TTkK.BOTTOM)

        if self._mbTop:
            self._cbTop.setChecked(True)
            self._btnTop.setEnabled(True)
        if self._mbBottom:
            self._cbBottom.setChecked(True)
            self._btnBottom.setEnabled(True)

        if self._mbTop:
            self._showEditor(ttk.TTkK.TOP)
        elif self._mbBottom:
            self._showEditor(ttk.TTkK.BOTTOM)

    def _enableMenuBar(self, enable, place):
        if enable:
            if place==ttk.TTkK.TOP:
                self._mbTop = mb = self._mbTop if self._mbTop else ttk.TTkMenuBarLayout()
            else:
                self._mbBottom = mb = self._mbBottom if self._mbBottom else ttk.TTkMenuBarLayout()
            self._widget.setMenuBar(mb, place)
        else:
            self._widget.setMenuBar(None, place)
        self._designer.weModified.emit()

    def _showEditor(self, place):
        if place==ttk.TTkK.TOP:
            self._btnTop.setChecked(True)
            self._btnBottom.setChecked(False)
            self.setTitle("MenuBar Editor (TOP)")
            mb = self._mbTop if self._mbTop else ttk.TTkMenuBarLayout()
        else:
            self._btnTop.setChecked(False)
            self._btnBottom.setChecked(True)
            self.setTitle("MenuBar Editor (BOTTOM)")
            mb = self._mbBottom if self._mbBottom else ttk.TTkMenuBarLayout()

        meL = _MenuItemEditor(mb._mbItems(ttk.TTkK.LEFT_ALIGN),   designer=self._designer)
        meC = _MenuItemEditor(mb._mbItems(ttk.TTkK.CENTER_ALIGN), designer=self._designer)
        meR = _MenuItemEditor(mb._mbItems(ttk.TTkK.RIGHT_ALIGN),  designer=self._designer)

        self._editorMenuBar.replaceItem(0,meL,title="Left")
        self._editorMenuBar.replaceItem(1,meC,title="Center")
        self._editorMenuBar.replaceItem(2,meR,title="Right")


    @staticmethod
    def spawnMenuBarEditor(designer):
        def _spawnMenuBarEditor(widget):
            menuBarEditor = MenuBarEditor(widget=widget, designer=designer)
            ttk.TTkHelper.overlay(None, menuBarEditor, 10, 5, toolWindow=True)
        return _spawnMenuBarEditor



# MIT License
#
# Copyright (c) 2023 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from random import randint

import TermTk as ttk
import ttkDesigner.app.superobj as so
from .superobj import SuperObject

class SuperLayout(ttk.TTkContainer):
    __slots__ = ('_lay', '_dropBorder', '_superRootWidget', '_selectable', '_designer')
    def __init__(self, lay, designer, *args, **kwargs):
        self._designer = designer
        self._lay = lay
        self._dropBorder = 0 # This property is used to exclude the border from the drop routine
        self._superRootWidget = kwargs.get('superRootWidget',False)
        self._selectable = kwargs.get('selectable', False)

        # kwargs['pos']  = (x,y) = lay.pos()
        # x,y = kwargs.get('pos',lay.pos())
        # kwargs['size'] = (w,h) = lay.size()
        # kwargs['layout'] = lay # .__class__()
        # self._lay.setGeometry(x,y,w,h)

        super().__init__(*args, **kwargs)

        self.getCanvas().setTransparent(True)
        # r,g,b = randint(0,0xFF),randint(0,0xFF),randint(0,0xFF)
        # self._layoutColor    = ttk.TTkColor.bg(f"#{r:02X}{g:02X}{b:02X}")
        self.setFocusPolicy(ttk.TTkK.ClickFocus)
        so.SuperWidget.toggleHighlightLayout.connect(self._toggleHighlightLayout)

    # TODO: Find a better way to handle this exception
    # It may require some major rewrite
    def hasControlWidget(self):
        return True

    def getSuperProperties(self):
        additions = {}
        exceptions = {}
        exclude = []
        return additions, exceptions, exclude

    @ttk.pyTTkSlot(bool)
    def _toggleHighlightLayout(self, state):
        so.SuperWidget._showLayout = state
        self.update()

    def setDropBorder(self, db):
        self._dropBorder = db

    def dropBorder(self):
        return self._dropBorder

    def dumpDict(self):
        children=[]
        for w in self.layout().children():
            layoutItemParams = {
                'row':w._row,
                'col':w._col,
                'rowspan':w._rowspan,
                'colspan':w._colspan,
            }
            children.append(w.widget().dumpDict()|layoutItemParams)
        ret = {'class': self.layout().__class__.__name__,
               # 'params' : SuperObject.dumpParams(self._lay),
               'params' : SuperObject.dumpParams(self.layout())|{'Geometry':self.geometry()},
               'children':children}
        return ret

    def updateAll(self):
        # self.resize(*(self._lay.size()))
        # self.move(*(self._lay.pos()))
        # self.setPadding(*(self._lay.getPadding()))
        # self.setMaximumSize(*(self._lay.maximumSize()))
        # self.setMinimumSize(*(self._lay.minimumSize()))
        self.update()

    def mousePressEvent(self, evt) -> bool:
        return self._selectable and not self._superRootWidget

    def pushSuperControlWidget(self):
        if self._superRootWidget or not self._selectable: return False
        scw = so.SuperControlWidget(self)
        ttk.TTkHelper.removeOverlay()
        ttk.TTkHelper.overlay(self, scw, -1,-1, forceBoundaries=False)

    def mouseReleaseEvent(self, evt) -> bool:
        if self._superRootWidget or not self._selectable: return False
        self.pushSuperControlWidget()
        self._designer.thingSelected.emit(self._lay,self)
        return True

    def mouseDragEvent(self, evt) -> bool:
        if self._superRootWidget or not self._selectable: return False
        drag = ttk.TTkDrag()
        data = self
        canvas = self.getCanvas()
        canvas.clean()
        self.paintEvent(canvas)
        ttk.TTkWidget._paintChildCanvas(canvas, self.layout(), self.layout().geometry(), self.layout().offset())
        drag.setHotSpot(evt.x, evt.y)
        drag.setPixmap(canvas)
        drag.setData(data)
        drag.exec()
        self.parentWidget().removeSuperWidget(self)
        return True

    def isRootWidget(self):
        return self._superRootWidget

    def makeRootWidget(self):
        self._superRootWidget = True

    def superChild(self):
        return self._lay

    @staticmethod
    def slFromLayout(layout: ttk.TTkLayout, *args, **kwargs):
        if 'lay' not in kwargs:
            kwargs |= {'lay':layout}
        if issubclass(type(layout),ttk.TTkVBoxLayout):
            sl = so.SuperLayoutVBox(*args, **kwargs)
        elif issubclass(type(layout),ttk.TTkHBoxLayout):
            sl = so.SuperLayoutHBox(*args, **kwargs)
        elif issubclass(type(layout),ttk.TTkGridLayout):
            sl = so.SuperLayoutGrid(*args, **kwargs)
        else:
            sl = so.SuperLayout(    *args, **kwargs)
        return sl

    def addSuperWidget(self, sw):
        self.layout().addWidget(sw)

    def removeSuperWidget(self, sw):
        self._lay.removeItem(self._lay)
        sc = sw.superChild()
        if issubclass(type(sc),ttk.TTkLayout):
            self._lay.removeItem(sc)
        else:
            self._lay.removeWidget(sc)
        self.layout().removeWidget(sw)
        self.layout().update()
        self.update()

    def isInDropBorder(self,x,y):
        if db := self._dropBorder:
            w,h = self.size()
            if not (db < x < w-1-db and db < y < h-1-db):
                return True
        return False

    def dropEvent(self, evt) -> bool:
        # Skip the drop event if dropborder is set and the cursor is along the edges
        if self.isInDropBorder(evt.x, evt.y): return False
        data = evt.data()
        hsx,hsy = evt.hotSpot()
        ttk.TTkLog.debug(f"Drop ({data.__class__.__name__}) -> pos={evt.pos()}")
        if issubclass(type(data),ttk.TTkLayout):
            _,__,w,h = data.geometry()
            x,y = evt.x-hsx, evt.y-hsy
            lay = ttk.TTkLayout(pos=(x,y), size=(w,h))
            sl = SuperLayout.slFromLayout(layout=data, designer=self._designer, lay=lay, pos=(x,y), size=(w,h), selectable=True)
            self.addSuperWidget(sl)
            self._lay.addItem(lay)
        elif issubclass(type(data), so.SuperLayout):
            sl = data
            self.addSuperWidget(sl)
            self._lay.addItem(sl._lay)
            sl.show()
            sl.move(evt.x-hsx, evt.y-hsy)
        elif issubclass(type(data), so.SuperWidgetContainer):
            sw = data
            self.addSuperWidget(sw)
            self._lay.addWidget(sw._wid)
            sl = sw._superLayout
            sw.move(evt.x-hsx, evt.y-hsy)
            sw.show()
        elif issubclass(type(data), so.SuperWidget):
            sw = data
            self.addSuperWidget(sw)
            self._lay.addWidget(sw._wid)
            sl = None
            sw.move(evt.x-hsx, evt.y-hsy)
            sw.show()
        elif issubclass(type(data),ttk.TTkTextEdit):
            self.addSuperWidget(sw := so.SuperWidget.swFromWidget(designer=self._designer, wid=data, pos=(evt.x-hsx, evt.y-hsy)))
            self._lay.addWidget(data)
            sw.move(evt.x-hsx, evt.y-hsy)
            sl = None
        elif issubclass(type(data),ttk.TTkContainer):
            self.addSuperWidget(sw := so.SuperWidget.swFromWidget(designer=self._designer, wid=data, pos=(evt.x-hsx, evt.y-hsy)))
            self._lay.addWidget(data)
            sw.move(evt.x-hsx, evt.y-hsy)
            sl = sw._superLayout
        elif issubclass(type(data),ttk.TTkWidget):
            self.addSuperWidget(sw := so.SuperWidget.swFromWidget(designer=self._designer, wid=data, pos=(evt.x-hsx, evt.y-hsy)))
            self._lay.addWidget(data)
            sw.move(evt.x-hsx, evt.y-hsy)
            sl = None
        else:
            return False

        self.layout().update()

        # set the Drop Border in case this layout auto resize
        if sl:
            if issubclass(type(self.layout()),ttk.TTkGridLayout):
                sl.setDropBorder(1)
            else:
                sl.setDropBorder(0)

        self.update()
        self._designer.weModified.emit()
        return True

    def move(self, x: int, y: int):
        w,h = self._lay.size()
        self._lay.setGeometry(x,y,w,h)
        return super().move(x, y)

    def resizeEvent(self, w, h):
        x,y = self._lay.pos()
        self._lay.setGeometry(x,y,w,h)
        return super().resizeEvent(w, h)

    def paintEvent(self, canvas):
        if self._selectable:
            if so.SuperWidget._showLayout:
                w,h = self.size()
                canvas.drawBox(pos=(0,0),size=(w,h), color=ttk.TTkColor.fg('#88DD88', modifier=ttk.TTkColorGradient(increment=+1)))
            else:
                w,h = self.size()
                canvas.drawBox(pos=(0,0),size=(w,h), color=ttk.TTkColor.fg('#223322', modifier=ttk.TTkColorGradient(increment=+1)))



# MIT License
#
# Copyright (c) 2023 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import TermTk as ttk

class SuperControlWidget(ttk.TTkResizableFrame):
    def __init__(self, wid, *args, **kwargs):
        self._wid = wid
        self._widPos = self._wid.pos()
        self._draggable = False
        self._mouseDelta = (0,0)
        kwargs['maxSize'] = [v+2 for v in wid.maximumSize()]
        kwargs['minSize'] = [v+2 for v in wid.minimumSize()]
        kwargs['size']    = [v+2 for v in wid.size()       ]
        super().__init__(*args, **kwargs)
        self.getCanvas().setTransparent(True)

    def _alignWidToPos(self, pos):
        x,y = self.pos()
        ox,oy = pos
        wx,wy = self._wid.pos()
        self._wid.move(wx+x-ox, wy+y-oy)
        self.update()
        return super().move(x,y)

    def resizeEvent(self, w, h):
        super().resizeEvent(w, h)
        self._wid.resize(w-2,h-2)
        self._wid._canvas.updateSize()

    def mouseMoveEvent(self, evt) -> bool:
        return True

    def mouseReleaseEvent(self, evt) -> bool:
        self._draggable = False
        return super().mouseReleaseEvent(evt)

    def mousePressEvent(self, evt):
        self._draggable = False
        self._mouseDelta = (evt.x, evt.y)
        w,h = self.size()
        x,y = evt.x, evt.y
        if x==0 or x==w-1 or y==0 or y==h-1:
            return super().mousePressEvent(evt)
        self._draggable = True
        return True

    def mouseDragEvent(self, evt):
        bkPos = self.pos()
        if self._draggable:
            x,y = self.pos()
            dx = evt.x-self._mouseDelta[0]
            dy = evt.y-self._mouseDelta[1]
            self.move(x+dx, y+dy)
            self._alignWidToPos(bkPos)
            return True
        ret = super().mouseDragEvent(evt)
        self._alignWidToPos(bkPos)
        return ret

    def keyEvent(self, evt):
        if evt.type == ttk.TTkK.SpecialKey:
            if ( evt.key in (ttk.TTkK.Key_Delete, ttk.TTkK.Key_Backspace) and
                 not self._wid.isRootWidget() ):
                self._wid.parentWidget().removeSuperWidget(self._wid)
                self._wid.close()
                self.close()
                self._wid._designer.weModified.emit()
                return True
            bkPos = self.pos()
            x,y = 0,0
            if   evt.key == ttk.TTkK.Key_Up:    y=-1
            elif evt.key == ttk.TTkK.Key_Down:  y= 1
            elif evt.key == ttk.TTkK.Key_Left:  x=-1
            elif evt.key == ttk.TTkK.Key_Right: x= 1
            else: return super().keyEvent(evt)
            if any((x,y)):
                self.move(bkPos[0]+x, bkPos[1]+y)
                self._alignWidToPos(bkPos)
            return True
        return super().keyEvent(evt)

    def paintEvent(self, canvas):
        w,h = self.size()
        self._wid.paintEvent(canvas)
        self._wid.paintChildCanvas()
        canvas.paintCanvas(
                self._wid.getCanvas(),
                (    1,     1, w, h), # geometry
                (    0,     0, w, h), # slice
                (    0,     0, w, h)) # bound
        canvas.drawBox(pos=(0,0),size=self.size())
        canvas.drawChar(pos=(  0,   0), char='▛')
        canvas.drawChar(pos=(w-1,   0), char='▜')
        canvas.drawChar(pos=(  0, h-1), char='▙')
        canvas.drawChar(pos=(w-1, h-1), char='▟')



# MIT License
#
# Copyright (c) 2023 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import TermTk as ttk
import ttkDesigner.app.superobj as so
from ttkDesigner.app.menuBarEditor import MenuBarEditor

class SuperWidgetFrame(so.SuperWidgetContainer):
    def getSuperProperties(self):
        additions, exceptions, exclude = super().getSuperProperties()
        additions |= {
            ttk.TTkFrame.__name__ : {
                'Menu Bar' : {
                    'get':  {'cb':MenuBarEditor.spawnMenuBarEditor(self._designer), 'type':'button', 'text':'Edit'},
                    'set':  {'cb':MenuBarEditor.spawnMenuBarEditor(self._designer), 'type':'button', 'text':'Edit'} },
            }
        }
        return additions, exceptions, exclude

    def dumpDict(self):
        def _dumpMenuBarItem(_mbi:ttk.TTkMenuButton):
            if not issubclass(type(_mbi),ttk.TTkMenuButton):
                return "spacer"

            ret = {'params': {
                        'Name':      _mbi.name(),
                        'ToolTip':   _mbi.toolTip().toAnsi(True),
                        'Text':      _mbi.text().toAnsi(True),
                        'Checkable': _mbi.isCheckable(),
                        'Checked':   _mbi.isChecked(),
                    }
                }
            if _sm := _dumpMenuBarItems(_mbi._submenu):
                ret |= {'submenu': _sm}
            return ret

        def _dumpMenuBarItems(_mbis):
            return [_dumpMenuBarItem(_i) for _i in _mbis]

        def _dumpMenuBar(_mb):
            ret = {}
            if _mbis := _mb._mbItems(ttk.TTkK.LEFT_ALIGN).children():
                ret |= {'left':_dumpMenuBarItems([_i.widget() for _i in _mbis])}
            if _mbis := _mb._mbItems(ttk.TTkK.CENTER_ALIGN).children():
                ret |= {'center':_dumpMenuBarItems([_i.widget() for _i in _mbis])}
            if _mbis := _mb._mbItems(ttk.TTkK.RIGHT_ALIGN).children():
                ret |= {'right':_dumpMenuBarItems([_i.widget() for _i in _mbis])}
            return ret

        ret = super().dumpDict()
        barTop    = self._wid.menuBar(ttk.TTkK.TOP)
        barBottom = self._wid.menuBar(ttk.TTkK.BOTTOM)
        if barTop or barBottom:
            ret |= {'menuBar' : {}}
            if barTop:
                ret['menuBar']['top'] = _dumpMenuBar(barTop)
            if barBottom:
                ret['menuBar']['bottom'] = _dumpMenuBar(barBottom)
        return ret



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

version = "0.38.0-a2"
name = "ttkDesigner"

print(f"Version: {version}")
print(f"Name: {name}")

setup(
    name=name,
    version=version,
    author='Eugenio Parodi',
    author_email='ceccopierangiolieugenio@googlemail.com',
    description='ttkDesigner is a terminal user interface designer for pyTermTk applications',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ceccopierangiolieugenio/pyTermTk",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Topic :: Terminals",
        "Topic :: Software Development :: User Interfaces"],
    include_package_data=False,
    packages=['ttkDesigner','ttkDesigner.app', 'ttkDesigner.app.superobj'],
    package_data={'ttkDesigner': ['tui/*']},
    python_requires=">=3.9",
    install_requires=[
        'pyTermTk>=0.38.0a',
        'pyperclip',
        'Pillow'],
    entry_points={
        'console_scripts': [
            'ttkDesigner = ttkDesigner:main',
        ],
    },
)